from __future__ import absolute_import
from ._dionysus import *
from . import plot     # make it available without an explicit import
from ._version import version_info, __version__

def closure(simplices, k):
    """Compute the `k`-skeleton of the closure of the list of `simplices`."""

    res = set()

    from    itertools   import combinations
    for s in simplices:
        for kk in range(1, k+2):
            for face in combinations(list(s), min(s.dimension() + 1, kk)):
                ss = Simplex(face)
                res.add(ss)

    return list(res)


def smooth(f, z, prime, show = False):
    """Smooth a given integer cocycle into a harmonic cocycle."""

    try:
        from scipy.sparse.linalg import lsqr
        from scipy.sparse        import csc_matrix
        import numpy as np
    except ImportError:
        raise ImportError("Unable to import lsqr from scipy.sparse.linalg. Have you installed scipy?")

    # Cocycle can be larger than D; we implicitly project it down

    data = []
    row  = []
    col  = []
    for i,s in enumerate(f):
        if s.dimension() == 1:
            for isb,sb in enumerate(s.boundary()):
                data.append(1. if isb % 2 == 0 else -1.)
                row.append(i)
                col.append(f.index(sb))

    row_max = max(row)           # x.index <= row_max condition below projects the cocycle to the filtration
    z_data = [x.element if x.element < prime/2 else x.element - prime for x in z if x.index <= row_max]
    z_row  = [x.index for x in z if x.index <= row_max]
    z_col  = [0 for x in z if x.index <= row_max]

    dim = max(row_max,max(col)) + 1
    D = csc_matrix((np.array(data), (np.array(row), np.array(col))), shape=(dim, dim))
    z = csc_matrix((z_data, (z_row, z_col)), shape=(dim, 1)).toarray()

    tol = 1e-10
    solution = lsqr(D, z, atol = tol, btol = tol, show = show)

    max_vrt = max(s[0] for s in f if s.dimension() == 0)

    vertex_values = [0. for _ in range(max_vrt + 1)]
    for i,x in enumerate(solution[0]):
        if f[i].dimension() == 0:
            vertex_values[f[i][0]] = x

    return vertex_values

def is_simplicial(f, report = False):
    """Check if filtration is simplicial. If `report = True`, output the problems."""
    simplicial = True
    for s in f:
        if len(s) != len(set(s)):
            if report:
                print("%s is not a simplex" % s)
                simplicial = False
            else:
                return False
        for sb in s.boundary():
            if sb not in f:
                if report:
                    print("%s in boundary of %s not found in the filtration" % (sb, s))
                    simplicial = False
                else:
                    return False
    return simplicial



import dionysus as d

def test_distance():
    dgm1 = d.Diagram([(1,2), (3,4), (1., float('inf'))])
    dgm2 = d.Diagram([(0,2), (3,5), (2., float('inf'))])
    dist = d.bottleneck_distance(dgm1,dgm2)
    assert dist == 1



import numpy as np
import dionysus as d

def test_issue39():
    dgm1 = np.loadtxt('data/issue39/dgm1.txt', delimiter=',')
    dgm2 = np.loadtxt('data/issue39/dgm2.txt', delimiter=',')
    dgm1 = d.Diagram(dgm1)
    dgm2 = d.Diagram(dgm2)
    dist = d.wasserstein_distance(dgm1,dgm2,q=5)



def plot_diagram(dgm, show=False, labels=False, ax=None,
                 line_style=None, pt_style=None):
    """
    Plot the persistence diagram.

    Arguments:
        dgm (Diagram): See for example `init_diagrams`.

    Keyword Arguments:
        show (bool): Display the plot. (Default: False)
        labels (bool): Set axis labels. (Default: False)
        ax (AxesSubplot): Axes that should be used for plotting (Default: None)
        pt_style (dict): argments passed to `ax.scatter` for style of points.
        line_style (dict): argments passed to `ax.plot` for style of diagonal line.
    """

    import matplotlib.pyplot as plt

    line_kwargs = {}
    pt_kwargs = {}
    if pt_style is not None:
        pt_kwargs.update(pt_style)
    if line_style is not None:
        line_kwargs.update(line_style)


    inf = float('inf')
    min_birth = min(p.birth for p in dgm if p.birth != inf)
    max_birth = max(p.birth for p in dgm if p.birth != inf)
    min_death = min(p.death for p in dgm if p.death != inf)
    max_death = max(p.death for p in dgm if p.death != inf)

    if ax is None:
        ax = plt.axes()
    ax.set_aspect('equal', 'datalim')

    min_diag = min(min_birth, min_death)
    max_diag = max(max_birth, max_death)
    ax.scatter([p.birth for p in dgm], [p.death for p in dgm], **pt_kwargs)
    ax.plot([min_diag, max_diag], [min_diag, max_diag], **line_kwargs)

    if labels:
        ax.set_xlabel('birth')
        ax.set_ylabel('death')

    ## clip the view
    #plt.axes().set_xlim([min_birth, max_birth])
    #plt.axes().set_ylim([min_death, max_death])

    if show:
        plt.show()

def plot_bars(dgm, order='birth', show=False, ax=None, **bar_style):
    """
    Plot the barcode.

    Arguments:
        dgm (Diagram): See for example `init_diagrams`.

    Keyword Arguments:
        order (str): How to sort the bars, either 'death' or 'birth'
                     (Default: 'birth')
        show (bool): Display the plot. (Default: False)
        ax (AxesSubplot): Axes that should be used for plotting (Default: None)
        **bar_style: Arguments passed to `ax.plot` for style of the bars.
                     (Defaults: color='b')
    """

    import matplotlib.pyplot as plt

    bar_kwargs = {'color': 'b'}
    bar_kwargs.update(bar_style)

    if order == 'death':
        generator = enumerate(sorted(dgm, key = lambda p: p.death))
    else:
        generator = enumerate(dgm)

    if ax is None:
        ax = plt.axes()

    for i,p in generator:
        ax.plot([p.birth, p.death], [i,i], **bar_kwargs)

    if show:
        plt.show()


def plot_diagram_density(dgm, lognorm=True, diagonal=True,
                         show=False, labels=False, ax=None, **hist_style):
    """
    Plot the histogram of point density.

    Arguments:
        dgm (Diagram): See for example `init_diagrams`.

    Keyword Arguments:
        bins (int): bins for histogram, see `ax.hist2d` (Default: 200)
        lognorm (bool): Use logarithmic norm (Default: True)
        diagonal (bool):  (Default: True)
        show (bool): Display the plot. (Default: False)
        labels (bool): Set axis labels. (Default: False)
        ax (AxesSubplot): Axes that should be used for plotting (Default: None)
        **hist_style: Arguments passed to `ax.hist2d` for style of the histogram.
            (Defaults: bins=200)
    """

    import matplotlib.pyplot as plt
    from   matplotlib.colors import LogNorm, Normalize

    hist_kwargs = {'bins': 200}
    hist_kwargs.update(hist_style)

    if lognorm:
        norm = LogNorm()
    else:
        norm = Normalize()

    inf = float('inf')
    min_birth = min(p.birth for p in dgm if p.birth != inf)
    #max_birth = max(p.birth for p in dgm if p.birth != inf)
    #min_death = min(p.death for p in dgm if p.death != inf)
    max_death = max(p.death for p in dgm if p.death != inf)

    if ax is None:
        _, ax = plt.subplots()

    hist2d, histx, histy, im = ax.hist2d([p.birth for p in dgm if p.birth != inf and p.death != inf], [p.death for p in dgm if p.birth != inf and p.death != inf], norm=norm, **hist_kwargs)
    ax.set_aspect('equal', 'datalim')
    if labels:
        ax.set_xlabel('birth')
        ax.set_ylabel('death')

    if diagonal:
        ax.plot([min_birth, max_death], [min_birth, max_death])

    ## clip the view
    #plt.axes().set_xlim([min_birth, max_birth])
    #plt.axes().set_ylim([min_death, max_death])

    if labels:
        plt.colorbar(im, ax=ax, label='overlap quantity')
    else:
        plt.colorbar(im, ax=ax)

    if show:
        plt.show()
